#ifndef __IMAGE_H__
#define __IMAGE_H__

#ifdef _WIN32
#include <GL\glut.h>
#else
#include <GL/glut.h>
#endif

typedef unsigned char BYTE;
typedef unsigned short int WORD;
typedef unsigned int DWORD;
typedef long int LONG;
typedef unsigned int UINT;

#ifdef _WIN32
typedef unsigned long QWORD;				// Windows 8 byte (???)
#else
typedef unsigned long long int QWORD;		// Unix 8 byte
#endif


#define TRUE	1
#define FALSE	0

#pragma pack(1)


/////////////////////////////////////////////////////////////////////////////
//
//	Universelle Textur Struktur
//
/////////////////////////////////////////////////////////////////////////////
struct ImageRec {
	int BPP;					// Bits Per Pixel
    int sizeX;					// Breite
	int sizeY;					// Hoehe
    unsigned char *data;		// RGB Daten (bzw. Indizes)
	unsigned char *Palette;		// Optionale Palette
public:
	// Konstruktor
	ImageRec ();
	
	// Destruktor
	~ImageRec ();
	
	// Konvertieren 8bit in 24bit Format
	void Convert8bitToRGB ();

	// Konvertieren 8bit in 32bit Format mit vorgegebene Alpha Wert
	void Convert8bitToRGBA (int Alpha);

	// Konvertieren 24bit in 32bit Format mit vorgegebene Alpha Wert
	void ConvertRGB2RGBA (int Alpha);
	
	// Stellt den Alpha Wert im Bild auf Alpha ein (nur wenn das Bild
	// im 32bit Format vorliegt)
	void SetAlpha (int Alpha);

	// Vertikales Drehen des Bildes
	bool VerticalFlip ();

	// Horizontales Drehen des Bildes
	bool HorizontalFlip ();
	
	// Tauscht rote und blaue Komponente (relevant fuer TGA)
	bool SwapRB ();
};



/******************************************************************************/
//
//	RGB Header Struktur
//
/******************************************************************************/
typedef struct _rawImageRec {
    unsigned short imagic;
    unsigned short type;
    unsigned short dim;
    unsigned short sizeX, sizeY, sizeZ;
    unsigned long min, max;
    unsigned long wasteBytes;
    char name[80];
    unsigned long colorMap;
    FILE *file;
    unsigned char *tmp, *tmpR, *tmpG, *tmpB;
    unsigned long rleEnd;
    GLuint *rowStart;
    GLint *rowSize;
} rawImageRec;


/******************************************************************************/
//
//	BMP Header Strukturen
//
/******************************************************************************/
typedef struct tagRGBQUAD {				/* rgbq */
	BYTE    rgbBlue;
	BYTE    rgbGreen;
	BYTE    rgbRed;
	BYTE    rgbReserved;
} RGBQUAD;

typedef struct tagBITMAPINFOHEADER {    /* bmih */
    DWORD   biSize;
    int    biWidth;
    int    biHeight;
    WORD    biPlanes;
    WORD    biBitCount;
    DWORD   biCompression;
    DWORD   biSizeImage;
    int    biXPelsPerMeter;
    int    biYPelsPerMeter;
    DWORD   biClrUsed;
    DWORD   biClrImportant;
} BITMAPINFOHEADER;

typedef struct tagBITMAPFILEHEADER {    /* bmfh */
    WORD    bfType;
    DWORD   bfSize;
    WORD    bfReserved1;
    WORD    bfReserved2;
    DWORD   bfOffBits;
} BITMAPFILEHEADER;

typedef struct tagBITMAPINFO {  		/* bmi */
    BITMAPINFOHEADER    bmiHeader;
    RGBQUAD            	bmiColors[1];
} BITMAPINFO;


/******************************************************************************/
//  TGA File Header
/******************************************************************************/
struct TGAHeader {                           
    BYTE   IdLength;            /* Image ID Field Length      */
    BYTE   CmapType;            /* Color Map Type             */
    BYTE   ImageType;           /* Image Type                 */
    /*
    ** Color Map Specification
    */
    BYTE   CmapIndex[2];        /* First Entry Index          */
    WORD   CmapLength;       /* Color Map Length           */
    BYTE   CmapEntrySize;       /* Color Map Entry Size       */
    /*
    ** Image Specification
    */
    WORD   X_Origin;            /* X-origin of Image          */
    WORD   Y_Origin;            /* Y-origin of Image          */
    WORD   ImageWidth;          /* Image Width                */
    WORD   ImageHeight;         /* Image Height               */
    BYTE   PixelDepth;          /* Pixel Depth                */
    BYTE   ImagDesc;            /* Image Descriptor           */
};

/******************************************************************************/
//
//	Prototypen der Funktionen
//
/******************************************************************************/

// Funktion zum Einlesen einer Textur. Falls sie erfolgreich eingelesen
// wurde, wird ein Pointer auf ein Objekt der Klasse ImageRec 
// zurueckgegeben.
ImageRec *ReadTexture (const char *File);

///////////////////////////////////////////////////////////////////////////////
//	Hilfsfunktionen
///////////////////////////////////////////////////////////////////////////////
ImageRec *LoadBMP (const char *filename,bool Verbose = TRUE) ;
ImageRec *LoadTGA (const char *File,bool Verbose = TRUE);
ImageRec *LoadRGB (const char *File,bool Verbose = TRUE);

static void ConvertShort (unsigned short *array, unsigned int length);
static void ConvertUint (GLuint *array, unsigned int length);
rawImageRec *RawImageOpen (const char *fileName,bool Verbose);
static void RawImageClose(rawImageRec *raw);
bool RawImageGetRow (rawImageRec *raw, unsigned char *buf, int y, int z);
bool RawImageGetData (rawImageRec *raw, ImageRec *final);

///////////////////////////////////////////////////////////////////////////////
//	Allgemeine Hilfsfunktionen
///////////////////////////////////////////////////////////////////////////////
int GetFileSize (char *path);
int GetFileSize (int FileDes);


#endif
